from .utils.RealiBoxTestBiz import RealiBoxTestBiz
from .RealiBoxRequest import RealiBoxRequest
from .JsonPath.JsonPath import JsonPath
from .JsonPath.Assertion import Assertion
from .utils.sub_string_template import sub_string_template


class RealiBoxTestBase(RealiBoxTestBiz):
    # 断言方法
    def assertion(self, value):
        return Assertion(self, value)

    # http请求方法
    def http_request(self, req_json):
        return RealiBoxRequest(self, req_json=req_json)

    # jsonpath解析
    def json_path(self, expr, json_obj=None):
        return JsonPath(self, expr, json_obj or self.current_response_object)

    # json内容提取并断言简化方法，该方法仅用于响应内容的json提取
    def extract_assert(self, json_expr, index, assert_args=()):
        '''
        json_expr: jsonpath解析字符串
        index：json解析结果索引提取
        assert_args: 断言参数 （excepted， assertFunction, desc）
        用于对响应数据提取 / 断言。返回提取结果
        '''
        # 实现字符串匹配断言方法
        # 执行逻辑：
        #   1、根据json_expr调用 jsonpath 获得的内容
        #   2、根据index提取内容
        # index 为 int 根据索引提取
        if isinstance(index, int):
            content = JsonPath(self, json_expr, self.current_response_object).index(index)
        # index 为 str
        elif isinstance(index, str):
            if index.upper() == 'ALL':
                content = JsonPath(self, json_expr, self.current_response_object).all()
        else:
            raise ValueError(''' params 'index' can only int or str''')
        # 无需断言直接返回提取结果
        if not assert_args:
            return content.get()
        excepted = assert_args[0]
        if isinstance(excepted, str):
            excepted = sub_string_template(excepted, self.V)
        desc = assert_args[-1] if len(assert_args) >= 3 else None
        return getattr(content, assert_args[1])(excepted, desc=desc)

    def assert_many(self, *args):
        '''extract_assert的批量处理方法
        参数为多个元组， 元组内结果如extract_assert方法参数。
        注意：这是个批量处理方法，不返回提取结果，只能用与批量断言
        '''
        for i in args:
            self.extract_assert(*i)

    # 设置用例属性
    def set_attr(self, attr, value):
        setattr(self.V, attr, value)

    # 批量设置用例属性
    def set_many_attrs(self, *args):
        if len(args) < 2:
            return
        args_list = list(args)
        value, attr = [args_list.pop() for i in range(2)]
        setattr(self.V, attr, value)
        return self.set_many_attrs(*args_list)

    # 读取用例属性
    def get_attr(self, attr):
        return getattr(self.V, attr)








